import json

import requests

from .exceptions import *


class CoreApi:
    def __init__(self, login=None, password=None, api_key=None, url_api=None):
        if url_api is None:
            url_api = "https://app.mymarilyn.ru/"

        self.login = login
        self.password = password
        self.api_key = api_key
        self.url_api = url_api

        self.header = {
            "Accept": "application/json",
            "X-API-Token": self.api_key,
        }

        self.http = requests.Session()
        self.http.headers.update({"Accept": "application/json"})
        if api_key is not None:
            self.http.headers.update({"X-API-Token": self.api_key, })

    def _login(self):
        """
        Login user -> get new API Token

        http put to auth, send login and password,
        response to json (token and live time)
        """
        data = {
            "email": self.login,
            "password": self.password,
            "ttl": 180
        }

        req = self.http.put(url=self.url_api + 'auth', json=data)

        if req.status_code != 200:
            raise LoginRequired('status code is not 200')

        req_json = req.json()
        if req_json.get('token') is not None:
            self.api_key = req_json['token']
            self.http.headers.update({"X-API-Token": self.api_key, })
        else:
            raise LoginRequired('api key is None')

    def auth(self):
        """
        test auth users and get new api_key or use old api_key
        :return: None
        """
        if not self.login:
            raise LoginRequired('Login is required to auth')

        if not self.password and not self.api_key:
            raise LoginRequired('password and api_key is None')

        if self.api_key:
            req = self.http.get(self.url_api + 'me')
            if req.status_code == 200:
                req_json = req.json()
                if self.login == req_json['email']:
                    return
                else:
                    raise LoginRequired('login is no equal email')
            else:
                raise LoginRequired('login or password is not correct (maubi error api)')
        else:
            self._login()

    def get_api(self):
        return MarilynApiMethod(self)

    def method(self, method, values=None):
        if self.http.headers.get("X-API-Account"):
            del self.http.headers["X-API-Account"]

        if values.get('x_api_acoount'):
            self.http.headers.update({
                "X-API-Account": str(values['x_api_acoount'])
            })
            del values['x_api_acoount']
        req = self.http.get(self.url_api + str(method), params=values)
        if req.status_code == 200:
            try:
                req_json = req.json()
            except json.decoder.JSONDecodeError:
                raise MethodReturn('return text is not JSON, text: {}'.format(req.text))
            return req_json
        else:
            raise MethodReturn('status code is not 200, info: {}'.format(req.text))


class MarilynApiMethod(object):
    __slots__ = ('_marilyn', '_method')

    def __init__(self, marilyn, method=None):
        self._marilyn = marilyn
        self._method = method

    def __getattr__(self, method):
        if '_' in method:
            m = method.split('_')
            method = m[0] + ''.join(i.title() for i in m[1:])

        return MarilynApiMethod(
            self._marilyn,
            (self._method + '/{}' if method == 'id' else ((self._method + '/' if self._method else '') + method))
        )

    def __call__(self, **kwargs):
        if kwargs.get('api_id_list'):
            self._method = self._method.format(*kwargs['api_id_list'])
            del kwargs['api_id_list']
        return self._marilyn.method(self._method, kwargs)



import unittest

import Marilyn

try:
    from .config import login, password, api_url
except Exception:
    login = password = api_url = None


class TestMarilynBase(unittest.TestCase):
    api_key = ''

    def test_auth(self):
        if api_url is None:
            return self.assertTrue(True)

        marilyn = Marilyn.CoreApi(login=login, password=password, url_api=api_url)

        try:
            marilyn.auth()
        except Marilyn.exceptions.LoginRequired:
            self.assertTrue(False)
        else:
            self.assertIsNotNone(marilyn.api_key)
            self.__class__.api_key = marilyn.api_key

    def test_auth_tests_token(self):
        if api_url is None:
            return self.assertTrue(True)

        marilyn = Marilyn.CoreApi(login=login, password=password, api_key=self.__class__.api_key, url_api=api_url)

        try:
            marilyn.auth()
        except Marilyn.exceptions.LoginRequired as err:
            raise err

        method_api = marilyn.get_api()
        user_info = method_api.me()
        self.assertEqual(user_info['email'], login)

    def test_auth_not_correct_login(self):
        if api_url is None:
            return self.assertTrue(True)

        marilyn = Marilyn.CoreApi(login="test@test.ru", password=password, api_key=self.__class__.api_key,
                                  url_api=api_url)

        try:
            marilyn.auth()
        except Marilyn.exceptions.LoginRequired:
            self.assertTrue(True)
        else:
            self.assertTrue(False)

    def test_login_is_none(self):
        if api_url is None:
            return self.assertTrue(True)

        marilyn = Marilyn.CoreApi(login=None, password=password, api_key=self.__class__.api_key, url_api=api_url)

        try:
            marilyn.auth()
        except Marilyn.exceptions.LoginRequired:
            self.assertTrue(True)
        else:
            self.assertTrue(False)

    def test_readme_code(self):
        if api_url is None:
            return self.assertTrue(True)

        marilyn = Marilyn.CoreApi(login=login, password=password, api_key=self.__class__.api_key, url_api=api_url)

        try:
            marilyn.auth()
        except Marilyn.exceptions.LoginRequired:
            exit()

        method_api = marilyn.get_api()
        user_info = method_api.me()

        self.assertEqual(user_info['email'], login)



import os
import re

from distutils.core import setup


def read_file(path):
    with open(os.path.join(os.path.dirname(__file__), path)) as fp:
        return fp.read()


def _get_version_match(content, value_name):
    regex = r"^{} = ['\"]([^'\"]*)['\"]".format(value_name)
    version_match = re.search(regex, content, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def get_value(value_name):
    path = os.path.join('Marilyn', '__init__.py')

    return _get_version_match(read_file(path), value_name)


setup(
    name='Marilyn',
    packages=['Marilyn'],  # this must be the same as the name above
    version=get_value('__version__'),
    description='Marilyn SDK',
    author=get_value('__author__'),
    author_email=get_value('__email__'),
    license='Apache License 2.0',
    url='https://github.com/skar404/marilyn/',  # use the URL to the github repo
    download_url='https://github.com/skar404/marilyn/archive/master.zip',  # I'll explain this in a second
    keywords=['marilyn', 'sdk', 'api'],  # arbitrary keywords

    install_requires=['requests'],

    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ]
)
